from itertools import chain

from django.forms import Media
from django.utils.html import format_html


class CSPAwareMedia(Media):
    def __init__(self, *args, **kwargs):
        if 'csp_nonce' in kwargs:
            self.csp_nonce = kwargs.pop('csp_nonce')

        super().__init__(*args, **kwargs)

    def render_js(self):
        if hasattr(self, 'csp_nonce'):
            # fmt: off
            return [
                format_html(
                    '<script type="text/javascript" src="{}" nonce="{}"></script>',
                    self.absolute_path(path), self.csp_nonce
                ) for path in self._js
            ]
            # fmt: on
        else:
            return super().render_js()

    def render_css(self):
        if hasattr(self, 'csp_nonce'):
            media = sorted(self._css)
            # fmt: off
            return chain.from_iterable([
                format_html(
                    '<link href="{}" type="text/css" media="{}" nonce="{}" rel="stylesheet">',
                    self.absolute_path(path), medium, self.csp_nonce
                ) for path in self._css[medium]
            ] for medium in media)
            # fmt: on
        else:
            return super().render_css()

    def __add__(self, other):
        combined = CSPAwareMedia(csp_nonce=self.csp_nonce)
        combined._css_lists = self._css_lists + other._css_lists
        combined._js_lists = self._js_lists + other._js_lists
        return combined


__all__ = ['CSPAwareMedia']



import types

from csp_helpers.classes import CSPAwareMedia
from csp_helpers.utils import get_nonce


def patched_render(self, name, value, attrs=None, renderer=None):
    context = self.get_context(name, value, attrs)
    context['csp_nonce'] = self.csp_nonce
    return self._render(self.template_name, context, renderer)


class CSPFormMixin:
    def __init__(self, *args, **kwargs):
        if 'csp_nonce' in kwargs:
            self.csp_nonce = kwargs.pop('csp_nonce')
            super().__init__(*args, **kwargs)

            for _name, field in self.fields.items():
                # monkeypatch the widget!
                field.widget.csp_nonce = self.csp_nonce
                field.widget.render = types.MethodType(patched_render, field.widget)

        else:
            super().__init__(*args, **kwargs)

    @property
    def media(self):
        """Return all media required to render the widgets on this form."""
        if hasattr(self, 'csp_nonce'):
            media = CSPAwareMedia(csp_nonce=self.csp_nonce)
            for field in self.fields.values():
                media = media + field.widget.media
            return media


class CSPViewMixin:
    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        nonce = get_nonce(self.request)
        if nonce is not None:
            kwargs['csp_nonce'] = nonce
        return kwargs



from django import template
from django.utils.safestring import mark_safe

from csp_helpers.utils import get_nonce_from_context

try:
    import webpack_loader
except ImportError:
    webpack_loader = None

register = template.Library()


def render_bundle_csp(context, bundle_name, extension=None, config='DEFAULT', attrs=''):
    nonce = get_nonce_from_context(context)
    if nonce is not None:
        attrs = ' '.join([attrs, f'nonce="{nonce}"'])

    tags = webpack_loader.utils.get_as_tags(bundle_name, extension=extension, config=config, attrs=attrs)
    return mark_safe('\n'.join(tags))


if webpack_loader is not None:
    register.simple_tag(render_bundle_csp, takes_context=True)



def get_nonce(request):
    """Return the CSP nonce from the request, supporting both django-csp and Django 6+ native CSP."""
    # django-csp middleware sets request.csp_nonce
    nonce = getattr(request, 'csp_nonce', None)
    if nonce is not None:
        return nonce

    # Django 6+ native CSP middleware sets request._csp_nonce
    try:
        from django.middleware.csp import get_nonce as _get_nonce

        return _get_nonce(request)
    except ImportError:
        return None


def get_nonce_from_context(context):
    """Return the CSP nonce from a template context, supporting both django-csp and Django 6+ native CSP."""
    # Django 6+ context processor sets csp_nonce directly in context
    nonce = context.get('csp_nonce')
    if nonce is not None:
        return nonce

    # Fall back to reading from the request object
    request = context.get('request')
    if request is not None:
        return get_nonce(request)

    return None



from itertools import chain

from django import template
from django.utils.html import format_html
from django.utils.safestring import mark_safe

from csp_helpers.utils import get_nonce_from_context

register = template.Library()


@register.simple_tag(takes_context=True)
def media_csp(context, _form):
    """
    A template tag that takes a Form object in a request context,
     and spits out the correct tags with CSP nonces.
    """
    nonce = get_nonce_from_context(context)
    tags = []

    # fmt: off
    for url in _form.media._js:
        tags.append(
            format_html(
                '<script type="text/javascript" src="{}" nonce="{}"></script>',
                _form.media.absolute_path(url), nonce
            )
        )

    tags += chain.from_iterable([
        format_html(
            '<link href="{}" type="text/css" media="{}" rel="stylesheet" nonce="{}">',
            _form.media.absolute_path(path), medium, nonce
        ) for path in _form.media._css[medium]
    ] for medium in sorted(_form.media._css))
    # fmt: on

    return mark_safe('\n'.join(tags))
